Module Array_Functions
    ' normalize the array to a miximum value of 1
    Public Sub Normalize(ByRef arrArray() As Single)
        Dim i As Integer
        Dim Max As Single
        Dim Min As Single
        Dim Diff As Single

        max = arrArray(0)
        min = arrArray(0)

        ' search the data for the maximum and minimum values
        For i = 1 To UBound(arrArray)
            If arrArray(i) > max Then
                max = arrArray(i)
            End If
            If arrArray(i) < min Then
                min = arrArray(i)
            End If
        Next i

        ' protect against a division by zero error
        Diff = max - min
        If Diff = 0 Then
            Diff = 1
        End If

        ' adjust the values
        For i = 0 To UBound(arrArray)
            arrArray(i) = (arrArray(i) - min) / Diff
        Next i
    End Sub

    ' normalize the array to a miximum value of 1
    Public Function Normalize(ByRef arrArray() As Single, ByVal MaxMin() As Single) As Single()
        Dim i As Integer
        Dim Diff As Single
        Dim Result() As Single
        ReDim Result(UBound(arrArray))

        ' protect against a division by zero error
        Diff = MaxMin(0) - MaxMin(1)
        If Diff = 0 Then
            Diff = 1
        End If

        ' adjust the values
        For i = 0 To UBound(Result)
            Result(i) = (arrArray(i) - MaxMin(1)) / Diff

            ' catch out of bounds entries
            If Result(i) > 1 Then
                Result(i) = 1
            ElseIf Result(i) < 0 Then
                Result(i) = 0
            End If
        Next i

        Normalize = Result
    End Function

    ' Determine the diameter of a bounding sphere
    Public Function BoundingSphere(ByRef arrPoints(,) As Single) As Single
        Dim i As Integer
        Dim dist As Single
        Dim dist2 As Single

        dist = (arrPoints(0, 0) ^ 2 + arrPoints(0, 1) ^ 2 + arrPoints(0, 2) ^ 2) ^ 0.5
        For i = 1 To UBound(arrPoints, 1)
            dist2 = (arrPoints(i, 0) ^ 2 + arrPoints(i, 1) ^ 2 + arrPoints(i, 2) ^ 2) ^ 0.5
            If dist2 > dist Then
                dist = dist2
            End If
        Next i

        BoundingSphere = 2 * dist

    End Function

    ' calculate normals to each point by averaging the normals for each element that point is a part of
    Public Sub CalNormals(ByRef arrPoints(,) As Single, ByRef arrCells()() As Double, ByRef arrNormals(,) As Single, ByVal worker As System.ComponentModel.BackgroundWorker, ByVal ProgressPercentage As Integer)
        Dim CellNormals(UBound(arrCells), 2) As Single
        Dim cross(2) As Single
        Dim p1(2) As Single
        Dim p2(2) As Single
        Dim p3(2) As Single
        Dim i As Integer
        Dim j As Integer
        Dim StepSize As Single

        StepSize = Math.Round(UBound(arrCells, 1) / 50)

        Dim temp As Single
        i = 0

        ' Determine the normal to each cells
        'For i = 0 To UBound(arrCells, 1) ' bounds of a for loop don't update when array dimension changes, causes failure if you remove an element
        While i <= UBound(arrCells)

            temp = UBound(arrCells, 1)

            p1(0) = arrPoints(arrCells(i)(0), 0)
            p1(1) = arrPoints(arrCells(i)(0), 1)
            p1(2) = arrPoints(arrCells(i)(0), 2)

            ' if there are less than 5 nodes, use the first three for normal calculation
            If UBound(arrCells(i)) < 5 Then
                p2(0) = arrPoints(arrCells(i)(1), 0)
                p2(1) = arrPoints(arrCells(i)(1), 1)
                p2(2) = arrPoints(arrCells(i)(1), 2)

                p3(0) = arrPoints(arrCells(i)(2), 0)
                p3(1) = arrPoints(arrCells(i)(2), 1)
                p3(2) = arrPoints(arrCells(i)(2), 2)
            Else    ' if there are 5 or more nodes, use the first, third, and fifth for normal calculation
                p2(0) = arrPoints(arrCells(i)(2), 0)
                p2(1) = arrPoints(arrCells(i)(2), 1)
                p2(2) = arrPoints(arrCells(i)(2), 2)

                p3(0) = arrPoints(arrCells(i)(4), 0)
                p3(1) = arrPoints(arrCells(i)(4), 1)
                p3(2) = arrPoints(arrCells(i)(4), 2)
            End If

            p2 = Vector_Subtract(p2, p1)
            p3 = Vector_Subtract(p3, p1)

            ' Determine the Normal
            cross = Cross_Product(p3, p2)

            ' If the normal is non-zero, the cell is ok
            If Vector_Length(cross) > 0 Then
                CellNormals(i, 0) = cross(0)
                CellNormals(i, 1) = cross(1)
                CellNormals(i, 2) = cross(2)
                i = i + 1
            Else    ' If not, throw out the cell
                arrCells(i) = arrCells(UBound(arrCells))
                ReDim Preserve arrCells(UBound(arrCells) - 1)
            End If

            If i Mod StepSize = 0 Then
                worker.ReportProgress(ProgressPercentage)
            End If
        End While

        ' initialize normal array
        ReDim arrNormals(UBound(arrPoints, 1), 2)

        For i = 0 To UBound(arrPoints, 1)
            arrNormals(i, 0) = 0
            arrNormals(i, 1) = 0
            arrNormals(i, 2) = 0
        Next i

        ' loop through all cells
        For i = 0 To UBound(arrCells)
            ' loop through all points in the cell
            p2(0) = CellNormals(i, 0)
            p2(1) = CellNormals(i, 1)
            p2(2) = CellNormals(i, 2)

            For j = 0 To UBound(arrCells(i))
                ' add the cell normal to all points contained in it
                p1(0) = arrNormals(arrCells(i)(j), 0)
                p1(1) = arrNormals(arrCells(i)(j), 1)
                p1(2) = arrNormals(arrCells(i)(j), 2)

                p3 = Vector_Add(p1, p2)

                arrNormals(arrCells(i)(j), 0) = p3(0)
                arrNormals(arrCells(i)(j), 1) = p3(1)
                arrNormals(arrCells(i)(j), 2) = p3(2)
            Next j

            If i Mod StepSize = 0 Then
                worker.ReportProgress(ProgressPercentage)
            End If
        Next i

        ' make normals into unit vectors
        For i = 0 To UBound(arrNormals, 1)
            p1(0) = arrNormals(i, 0)
            p1(1) = arrNormals(i, 1)
            p1(2) = arrNormals(i, 2)

            If Vector_Length(p1) > 0 Then
                p1 = Vector_Unit(p1)

                arrNormals(i, 0) = p1(0)
                arrNormals(i, 1) = p1(1)
                arrNormals(i, 2) = p1(2)
            End If
        Next i

    End Sub

    ' find the maximum and minimum values of a vector
    Public Function FindMaxMin(ByRef Array() As Single) As Single()
        Dim MaxMin(1) As Single
        Dim i As Integer

        MaxMin(0) = Array(0)
        MaxMin(1) = Array(0)

        For i = 1 To UBound(Array)
            If Array(i) < MaxMin(1) Then
                MaxMin(1) = Array(i)
            ElseIf Array(i) > MaxMin(0) Then
                MaxMin(0) = Array(i)
            End If
        Next

        FindMaxMin = MaxMin
    End Function

    ' find the index of minimum value of a vector
    Public Function FindMinIndex(ByRef Array() As Single) As Single
        Dim MinIndex As Single
        Dim i As Integer

        MinIndex = 0

        For i = 1 To UBound(Array)
            If Array(i) < Array(MinIndex) Then
                MinIndex = i
            End If
        Next

        FindMinIndex = MinIndex
    End Function

    ' find the index of minimum value of a vector
    ' only indices conatined in the CheckMe array are checked
    Public Function FindMinIndex(ByRef Array() As Single, ByRef ValidIndices() As Single) As Single
        Dim MinIndex As Single
        Dim i As Integer

        MinIndex = ValidIndices(0)

        For i = 1 To UBound(ValidIndices)
            If Array(ValidIndices(i)) < Array(MinIndex) Then
                MinIndex = ValidIndices(i)
            End If
        Next

        FindMinIndex = MinIndex
    End Function

    ' sort the array lowest to highest along the specified axis
    Public Function BubbleSort(ByVal array(,) As Single, ByVal index As Integer) As Single(,)
        Dim i, j As Single
        Dim temp() As Single

        For i = LBound(array, 1) To UBound(array, 1) - 1
            If array(i, index) > array(i + 1, index) Then
                ReDim temp(UBound(array, 2))
                For j = LBound(array, 2) To UBound(array, 2)
                    temp(j) = array(i, j)
                Next
                For j = LBound(array, 2) To UBound(array, 2)
                    array(i, j) = array(i + 1, j)
                Next
                For j = LBound(array, 2) To UBound(array, 2)
                    array(i + 1, j) = temp(j)
                Next
                i = LBound(array, 1) - 1
            End If
        Next

        BubbleSort = array
    End Function
End Module
