Imports frmmain

Module Critical_Points
    Public Function FindHotSpots(ByVal worker As System.ComponentModel.BackgroundWorker, ByVal HotSpotMin As Single, ByVal HotSpotSensitivity As Single, ByVal ColorMode As Integer) As Single()
        Dim HotSpots() As Single                        ' list of hot spot node numbers
        Dim arrUnassignedNodes() As Single              ' list of all nodes not assigned to a hot spot
        ReDim arrUnassignedNodes(UBound(arrAdjIntensity, 1))
        Dim i, j As Single

        Dim MyIntensity() As Single
        ReDim MyIntensity(UBound(arrAdjIntensity))

        For i = 0 To UBound(MyIntensity)
            If ColorMode Then
                MyIntensity(i) = arrAdjIntensity(i)
            Else
                MyIntensity(i) = 1 - arrAdjIntensity(i)
            End If
        Next

        ' Fill the node list with the index of each node
        j = 0
        For i = 0 To UBound(arrUnassignedNodes)
            If MyIntensity(i) <= HotSpotMin Then
                arrUnassignedNodes(j) = i
                j = j + 1
            End If
        Next
        ReDim Preserve arrUnassignedNodes(j - 1)

        Dim arrReducedCells()() As Double
        arrReducedCells = ReduceCells(MyIntensity, HotSpotMin)

        ReDim HotSpots(-1)

        ' continue looking for hotspots as long as the minimum is below threshold
        ' if the value is -1, the use hasn't choosen a data set.  Don't search for hotspots
        While UBound(arrUnassignedNodes) >= 0
            ReDim Preserve HotSpots(UBound(HotSpots) + 1)
            HotSpots(UBound(HotSpots)) = FindMinIndex(MyIntensity, arrUnassignedNodes)

            ' find all nodes associated with that hot spot
            Dim arrNodeList() As Single                     ' list of nodes associated with this hot spot

            ' add the minimum to the node list and remove it from the unassigned nodes
            ReDim arrNodeList(0)
            arrNodeList(0) = HotSpots(UBound(HotSpots))
            RemoveFromArray(InArray(HotSpots(UBound(HotSpots)), arrUnassignedNodes), arrUnassignedNodes)

            ' expand the nodelist around the minimum
            i = 0

            ' expand the nodelist around any nodes added
            ' keep going until no new nodes are added

            While i <= UBound(arrNodeList)
                ExpandHotSpot(arrNodeList(i), arrUnassignedNodes, arrNodeList, arrReducedCells, MyIntensity, HotSpotSensitivity)
                i = i + 1

                worker.ReportProgress(1 - UBound(arrUnassignedNodes) / j, UBound(HotSpots))
                If worker.CancellationPending Then
                    Exit While
                End If
            End While
            If worker.CancellationPending Then
                Exit While
            End If
        End While

        ' return the list of hot spot nodes.
        FindHotSpots = HotSpots
    End Function

    ' expand the hotspot to all neighbors of the specified node
    Private Sub ExpandHotSpot(ByVal NodeIndex As Single, ByRef arrUnassignedNodes() As Single, ByRef arrNodeList() As Single, ByRef arrCells()() As Double, ByRef arrAdjIntensity() As Single, ByVal HotSpotSensitivity As Single)
        Dim i, j, k As Single
        Dim tmpIndex As Single

        ' determine the intensity of this point to help determine where the hotspot ends
        Dim minIntensity As Single
        minIntensity = arrAdjIntensity(NodeIndex) - HotSpotSensitivity

        ' search through the cells for all nodes connected to the current node
        For i = 0 To UBound(arrCells, 1)
            ' search each point in the cell
            For j = 0 To UBound(arrCells(i))
                ' if the cell contains the node of interest
                ' check the node before and the node after in the cell
                If arrCells(i)(j) = NodeIndex Then
                    ' check node before
                    If j = 0 Then
                        k = UBound(arrCells(i))
                    Else
                        k = j - 1
                    End If

                    ' if it is in, add it to the nodelist and remove from the unassigned node list
                    If NodeCheck(arrAdjIntensity(arrCells(i)(k)), minIntensity) Then
                        tmpIndex = InArray(arrCells(i)(k), arrUnassignedNodes)
                        ' if the node is still unassigned
                        If tmpIndex >= 0 Then
                            ' add it to the node list and remove from the unassigned nodes array
                            ReDim Preserve arrNodeList(UBound(arrNodeList) + 1)
                            arrNodeList(UBound(arrNodeList)) = arrCells(i)(k)
                            RemoveFromArray(tmpIndex, arrUnassignedNodes)
                        End If
                    End If

                    ' check node after
                    If j = UBound(arrCells(i)) Then
                        k = 0
                    Else
                        k = j + 1
                    End If

                    ' if it is in, add it to the nodelist and remove from the unassigned node list
                    If NodeCheck(arrAdjIntensity(arrCells(i)(k)), minIntensity) Then
                        tmpIndex = InArray(arrCells(i)(k), arrUnassignedNodes)
                        ' if the node is still unassigned
                        If tmpIndex >= 0 Then
                            ' add it to the node list and remove from the unassigned nodes array
                            ReDim Preserve arrNodeList(UBound(arrNodeList) + 1)
                            arrNodeList(UBound(arrNodeList)) = arrCells(i)(k)
                            RemoveFromArray(tmpIndex, arrUnassignedNodes)
                        End If
                    End If

                    ' stop searching this cell
                    Exit For
                End If
            Next j

        Next i
    End Sub

    ' check an individual node to see if it belongs in this hotspot
    Private Function NodeCheck(ByVal curIntensity As Single, ByVal refIntensity As Single) As Boolean
        Dim result As Boolean
        result = False

        ' if the intensity is below threshold (the point is not blue) or
        ' if the intensity is greater than the reference intensity (intensity is increasing in this direction)
        ' than the node is in
        If curIntensity > refIntensity Then
            result = True
        End If

        NodeCheck = result
    End Function

    ' check if a value is in an array
    ' return index if found, -1 if not
    Private Function InArray(ByVal Node As Single, ByRef NodeArray() As Single) As Single
        Dim i As Single
        Dim result As Single = -1

        For i = LBound(NodeArray) To UBound(NodeArray)
            If NodeArray(i) = Node Then
                result = i
                Exit For
            End If
        Next

        InArray = result
    End Function

    ' remove the given index from the given array
    ' order is not preserved
    Private Sub RemoveFromArray(ByVal Index As Single, ByRef NodeArray() As Single)
        Dim result As Boolean = False

        NodeArray(Index) = NodeArray(UBound(NodeArray))
        ReDim Preserve NodeArray(UBound(NodeArray) - 1)
    End Sub

    ' reduce cells array to only those cells that have intensity values below the minimum
    Private Function ReduceCells(ByRef arrAdjIntensity() As Single, ByVal HotSpotMin As Single) As Double()()
        Dim result()() As Double
        Dim i As Double
        Dim j, k, l As Integer

        result = New Double(0)() {}

        l = -1
        For i = LBound(arrCells, 1) To UBound(arrCells, 1)
            For j = LBound(arrCells(i)) To UBound(arrCells(i))
                If arradjintensity(arrCells(i)(j)) <= HotSpotMin Then
                    l = l + 1
                    ReDim Preserve result(l)
                    result(l) = New Double(UBound(arrCells(i))) {}
                    For k = 0 To UBound(arrCells(i))
                        result(l)(k) = arrCells(i)(k)
                    Next
                    Exit For
                End If
            Next
        Next


        ReduceCells = result
    End Function
End Module