Imports System
Imports System.IO
Imports System.Math
Imports frmmain



Module FEA_FileIO


    ' Read in a model file generated by the FEA anaysis
    ' www.efatigue.com
    Public Function ReadFile(ByVal strFilename As String, ByVal worker As System.ComponentModel.BackgroundWorker, ByVal ProgressPercentage As Integer) As Int16
        Dim intError As Int16
        Dim StepSize As Single
        intError = 1

        ' read through the data file extracting data
        Try
            ' Create an instance of StreamReader to read from a file.
            ' The using statement also closes the StreamReader.
            Using sr As New StreamReader(strFilename)
                Dim strLine As String
                Dim booFound As Boolean
                Dim NumPoints As Integer
                Dim NumCells As Double

                ''''''''
                ' Verify the file
                strLine = sr.ReadLine
                If strLine = "# vtk DataFile Version 2.0" Then


                    ''''''''''''''''''
                    ' Find where the points start in the file

                    booFound = False

                    Do
                        strLine = sr.ReadLine()
                        If InStr(strLine, "POINTS", vbTextCompare) Then
                            booFound = True
                        End If
                    Loop Until (strLine Is Nothing) Or booFound

                    ' set errorflag if you did not find values
                    If Not (booFound) Then
                        intError = 1020
                        Exit Try

                        ' else, go ahead and collect the points
                    Else
                        Dim i As Integer
                        Dim j As Integer
                        Dim arrLine() As String
                        arrLine = Split(strLine, " ")
                        NumPoints = arrLine(1) - 1

                        ReDim arrPoints(NumPoints, 2)

                        StepSize = Ceiling(NumPoints / 50)

                        For i = 0 To NumPoints
                            strLine = sr.ReadLine()
                            arrLine = Split(strLine, " ")
                            For j = 0 To 2
                                arrPoints(i, j) = arrLine(j)
                            Next j

                            If i Mod StepSize = 0 Then
                                worker.ReportProgress(ProgressPercentage)
                            End If
                        Next i
                    End If



                    '''''''''''''''''
                    ' Find where the cells start in the file

                    booFound = False

                    Do
                        strLine = sr.ReadLine()
                        If InStr(strLine, "POLYGONS", vbTextCompare) Then
                            booFound = True
                        End If
                    Loop Until (strLine Is Nothing) Or booFound

                    ' set errorflag if you did not find values
                    If Not (booFound) Then
                        intError = 1021
                        Exit Try

                        ' else, go ahead and collect the cells
                    Else
                        Dim i As Integer
                        Dim j As Integer
                        Dim arrLine() As String
                        arrLine = Split(strLine, " ")
                        NumCells = arrLine(1) - 1

                        ' set length of arrCells
                        arrCells = New Double(NumCells)() {}

                        StepSize = Ceiling(NumCells / 50)

                        For i = 0 To NumCells
                            strLine = sr.ReadLine()
                            arrLine = Split(strLine, " ")

                            arrCells(i) = New Double(arrLine(0) - 1) {}
                            For j = 0 To UBound(arrCells(i))
                                arrCells(i)(j) = arrLine(j + 1)
                            Next j

                            If i Mod StepSize = 0 Then
                                worker.ReportProgress(ProgressPercentage)
                            End If
                        Next i
                    End If
                Else
                    MsgBox("Invalid file.  Select a valid eFatigue.com model file.", MsgBoxStyle.Critical, "Invalid File")
                    intError = 1023
                End If

            End Using

        Catch e As Exception
            ' Let the user know what went wrong.
            intError = 1023
            MsgBox(e.Message)
        End Try

        ReadFile = intError

    End Function

    ' Read in a model file generated by the FEA anaysis
    ' www.efatigue.com
    Public Function ReadIntensity(ByVal strFilename As String, ByVal strPlotType As String, ByVal ArrSize As Integer) As Int16
        Dim intError As Int16
        intError = 1

        If strPlotType = "None" Then
            Dim i As Integer
            ReDim arrIntensity(ArrSize)

            For i = 0 To ArrSize
                arrIntensity(i) = -1
            Next
        Else


            ' read through the data file extracting data
            Try
                ' Create an instance of StreamReader to read from a file.
                ' The using statement also closes the StreamReader.
                Using sr As New StreamReader(strFilename)
                    Dim strLine As String
                    Dim booFound As Boolean


                    '''''''''''''''''
                    ' Find where the intensity values start in the file

                    booFound = False

                    Do
                        strLine = sr.ReadLine()
                        If InStr(strLine, strPlotType, vbTextCompare) Then
                            booFound = True
                        End If
                    Loop Until (strLine Is Nothing) Or booFound

                    ' set errorflag if you did not find values
                    If Not (booFound) Then
                        intError = 1022
                        Exit Try

                        ' else, go ahead and collect the intensities
                    Else
                        Dim i As Integer
                        ReDim arrIntensity(ArrSize)
                        strLine = sr.ReadLine() ' discard next line

                        For i = 0 To ArrSize
                            strLine = sr.ReadLine()
                            arrIntensity(i) = strLine

                        Next i
                    End If

                End Using
            Catch e As Exception
                ' Let the user know what went wrong.
                intError = 1023
                MsgBox(e.Message)
            End Try

        End If

        ReadIntensity = intError

    End Function

    ' Read in a plot modes in the model file generated by the FEA anaysis
    ' www.efatigue.com
    Public Function ReadModes(ByVal strFilename As String) As String()
        Dim arrModes(0) As String
        arrModes(0) = "None"

        ' read through the data file extracting data
        Try
            ' Create an instance of StreamReader to read from a file.
            ' The using statement also closes the StreamReader.
            Using sr As New StreamReader(strFilename)
                Dim strLine As String
                Dim temp As Object

                '''''''''''''''''
                ' Find the plot modes in the file

                Do
                    strLine = sr.ReadLine()
                    If InStr(strLine, "SCALARS", vbTextCompare) Then

                        temp = Split(strLine, " ")
                        ReDim Preserve arrModes(UBound(arrModes) + 1)
                        arrModes(UBound(arrModes)) = temp(1)
                    End If
                Loop Until (strLine Is Nothing)

            End Using
        Catch e As Exception
            ' Let the user know what went wrong.
            MsgBox(e.Message)
        End Try

        ReadModes = arrModes

    End Function

    ' Adjust the data points to center the model on the origin
    Public Sub CenterModel(ByRef arrPoints(,) As Single)
        Dim i As Integer

        ' determine max's and mins
        Dim arrMaxs(2) As Single
        Dim arrMins(2) As Single

        arrMaxs(0) = arrPoints(0, 0)
        arrMaxs(1) = arrPoints(0, 1)
        arrMaxs(2) = arrPoints(0, 2)
        arrMins(0) = arrPoints(0, 0)
        arrMins(1) = arrPoints(0, 1)
        arrMins(2) = arrPoints(0, 2)

        For i = 1 To UBound(arrPoints, 1)
            ' x values
            If arrPoints(i, 0) > arrMaxs(0) Then
                arrMaxs(0) = arrPoints(i, 0)
            End If
            If arrPoints(i, 0) < arrMins(0) Then
                arrMins(0) = arrPoints(i, 0)
            End If

            ' y values
            If arrPoints(i, 1) > arrMaxs(1) Then
                arrMaxs(1) = arrPoints(i, 1)
            End If
            If arrPoints(i, 1) < arrMins(1) Then
                arrMins(1) = arrPoints(i, 1)
            End If

            ' z values
            If arrPoints(i, 2) > arrMaxs(2) Then
                arrMaxs(2) = arrPoints(i, 2)
            End If
            If arrPoints(i, 2) < arrMins(2) Then
                arrMins(2) = arrPoints(i, 2)
            End If
        Next i

        Dim arrOffsets(2) As Single

        arrOffsets(0) = (arrMaxs(0) + arrMins(0)) / 2
        arrOffsets(1) = (arrMaxs(1) + arrMins(1)) / 2
        arrOffsets(2) = (arrMaxs(2) + arrMins(2)) / 2

        ' make the changes to center the plot
        For i = 0 To UBound(arrPoints, 1)
            arrPoints(i, 0) = arrPoints(i, 0) - arrOffsets(0)
            arrPoints(i, 1) = arrPoints(i, 1) - arrOffsets(1)
            arrPoints(i, 2) = arrPoints(i, 2) - arrOffsets(2)
        Next i
    End Sub

    ' Determine the initial zoom factor based on the diameter of the bounding sphere
    Public Function SetZoom(ByRef arrpoints(,) As Single) As Single
        Dim D As Single
        D = BoundingSphere(arrpoints)

        SetZoom = Round(1.5 * D, 2)
    End Function

    Public Function ValidateData(ByVal data As String) As Int16
        Dim result As Int16
        result = 0

        If IsNumeric(data) Then
            result = 1
        End If

        ValidateData = result
    End Function

    Public Function SeekNodeValues(ByVal strFilename As String, ByRef PlotModes() As String, ByRef result(,) As Single) As Int16
        Dim intError As Int16
        intError = 1

        Dim j, k As Integer
        Dim i As Single
        Dim strLine As String
        Dim booFound As Boolean

        ' Create an instance of StreamReader to read from a file.
        ' The using statement also closes the StreamReader.
        'Try
        Using sr As New StreamReader(strFilename)
            ' read through the data file extracting data
            ' look up a value for each plot mode
            For j = 1 To UBound(PlotModes)
                ' Find where the intensity values start in the file
                booFound = False
                Do
                    strLine = sr.ReadLine()
                    If InStr(strLine, PlotModes(j), vbTextCompare) Then
                        booFound = True
                    End If
                Loop Until (strLine Is Nothing) Or booFound

                ' set errorflag if you did not find values
                If Not (booFound) Then
                    intError = 1022
                    'Exit Try

                    ' else, get the intensity for the node of interest
                Else
                    k = 0
                    strLine = sr.ReadLine() ' throw away the title row
                    For i = 0 To result(UBound(result, 1), 0) ' go through every node
                        strLine = sr.ReadLine()
                        If i = result(k, 0) Then
                            result(k, j) = strLine
                            k = k + 1
                        End If
                    Next i
                End If
            Next j
        End Using
        'Catch e As Exception
        '    ' Let the user know what went wrong.
        '    intError = 1023
        '    MsgBox(e.Message)
        'End Try


        SeekNodeValues = intError

    End Function
End Module