Imports Tao.OpenGl
Imports Tao.Platform.Windows
Imports System.Math


Module OpenGL_Common

    Public Sub InitializeOpenGL(ByVal w As Single, ByVal h As Single, ByVal zoom As Single, ByVal Light_Param As Single, ByVal BkgColor As rgb)
        ' Color to clear color buffer to.
        Gl.glClearColor(BkgColor.R, BkgColor.G, BkgColor.B, 0.0)

        ' Depth to clear depth buffer to; type of test.
        Gl.glClearDepth(1.0)
        Gl.glDepthFunc(Gl.GL_LESS)

        ' Enables Smooth Color Shading; try GL_FLAT for (lack of) fun.
        Gl.glShadeModel(Gl.GL_SMOOTH)

        ' Load up the correct perspective matrix; using a callback directly.
        ResizeScene(w, h, zoom)

        ' Set up a light, turn it on.
        Dim light()() As Single
        Dim light_pos(4) As Single

        light = LightSettings(Light_Param)
        light_pos(0) = 0
        light_pos(1) = 0
        light_pos(2) = -zoom
        light_pos(3) = 0.33

        Gl.glLightfv(Gl.GL_LIGHT1, Gl.GL_POSITION, light_pos)
        Gl.glLightfv(Gl.GL_LIGHT1, Gl.GL_AMBIENT, light(0))
        Gl.glLightfv(Gl.GL_LIGHT1, Gl.GL_DIFFUSE, light(1))
        Gl.glEnable(Gl.GL_LIGHT1)

        ' A handy trick -- have surface material mirror the color.
        'Gl.glColorMaterial(Gl.GL_FRONT_AND_BACK, Gl.GL_AMBIENT_AND_DIFFUSE)
        Gl.glEnable(Gl.GL_COLOR_MATERIAL)
    End Sub


    ' Main OpenGL Drawing function
    Public Sub DrawScene(ByVal hDC As System.IntPtr, ByRef arrPoints(,) As Single, ByVal arrCells()() As Double, ByRef arrIntensity() As Single, ByRef arrNormals(,) As Single, ByVal Zoom As Single, ByVal Xshift As Single, ByVal Yshift As Single, ByVal Plot_Grid As Boolean, ByVal Plot_model As Boolean, ByVal HotSpot As Single, ByVal HotSpotRadius As Single, Optional ByVal ColorMode As Integer = 1)
        Gl.glDisable(Gl.GL_TEXTURE_2D)
        Gl.glEnable(Gl.GL_LIGHTING)
        Gl.glBlendFunc(Gl.GL_SRC_ALPHA, Gl.GL_ONE_MINUS_SRC_ALPHA)
        Gl.glEnable(Gl.GL_DEPTH_TEST)

        ' Set camera position and viewing angle
        Gl.glMatrixMode(Gl.GL_MODELVIEW)
        Gl.glLoadIdentity()
        Gl.glClear(Gl.GL_COLOR_BUFFER_BIT)
        Gl.glClear(Gl.GL_DEPTH_BUFFER_BIT)

        ' Try using glRotate
        'Glu.gluLookAt(0.0, 0.0, Zoom, 0.0, 0.0, 0.0, 0, 1, 0)
        'Gl.glPushMatrix()
        Gl.glTranslatef(Xshift, Yshift, -Zoom)

        ' Start drawing polygons
        Gl.glEnable(Gl.GL_POLYGON_OFFSET_FILL)
        Gl.glPolygonOffset(1.0, 1.0)
        If Plot_model Then
            Model(arrPoints, arrCells, arrIntensity, arrNormals, ColorMode)
        End If
        If Plot_Grid Then
            Grid(arrPoints, arrCells, arrIntensity, arrNormals, ColorMode)
        End If
        If HotSpot <> Nothing Then
            Dim HotSpotCoord As New Coordinate(arrPoints(HotSpot, 0), arrPoints(HotSpot, 1), arrPoints(HotSpot, 2))
            'Dim arrcolor(2) As Single
            'arrcolor = ColorCode(arrIntensity(HotSpot), ColorMode)
            'Gl.glColor3f(arrcolor(0), arrcolor(1), arrcolor(2))
            'Gl.glColor3f(1, 1, 1)   'white
            Gl.glColor3f(0, 0, 0)   'black
            Sphere(HotSpotCoord, HotSpotRadius)
        End If
        'Triangle()

        Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, Gl.GL_FILL)
        Gl.glDisable(Gl.GL_POLYGON_OFFSET_FILL)

        'Gl.glPopMatrix()
        'Gl.glPopMatrix()
        'Gl.glFlush()
        Gdi.SwapBuffers(hDC)
    End Sub

    ' OpenGL commands to draw the model
    Private Sub Model(ByRef arrPoints(,) As Single, ByRef arrCells()() As Double, ByRef arrIntensity() As Single, ByRef arrNormals(,) As Single, Optional ByVal ColorMode As Integer = 1)
        Dim i As Integer
        Dim j As Integer
        Dim arrColor(2) As Single


        For i = 0 To UBound(arrCells)
            Gl.glBegin(Gl.GL_POLYGON)
            For j = 0 To UBound(arrCells(i))
                Gl.glNormal3f(arrNormals(arrCells(i)(j), 0), arrNormals(arrCells(i)(j), 1), arrNormals(arrCells(i)(j), 2))
                arrColor = ColorCode(arrIntensity(arrCells(i)(j)), ColorMode)
                Gl.glColor3f(arrColor(0), arrColor(1), arrColor(2))
                Gl.glVertex3f(arrPoints(arrCells(i)(j), 0), arrPoints(arrCells(i)(j), 1), arrPoints(arrCells(i)(j), 2))
            Next j
            Gl.glEnd()
        Next i


    End Sub

    ' OpenGL commands to draw the grid
    Private Sub Grid(ByRef arrPoints(,) As Single, ByRef arrCells()() As Double, ByRef arrIntensity() As Single, ByRef arrNormals(,) As Single, Optional ByVal ColorMode As Integer = 1)
        Dim i As Integer
        Dim j As Integer

        ' Start drawing grid lines
        Gl.glMatrixMode(Gl.GL_PROJECTION)
        Gl.glDisable(Gl.GL_LINE_SMOOTH)
        Gl.glDisable(Gl.GL_LIGHTING)

        Gl.glColor3f(0.0, 0.0, 0.0)                           ' black lines
        Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, Gl.GL_LINE)
        For i = 0 To UBound(arrCells)

            Gl.glBegin(Gl.GL_POLYGON)
            For j = 0 To UBound(arrCells(i))
                Gl.glVertex3f(arrPoints(arrCells(i)(j), 0), arrPoints(arrCells(i)(j), 1), arrPoints(arrCells(i)(j), 2))
            Next j
            Gl.glEnd()
        Next i

    End Sub

    ' OpenGL commands to draw a triangle
    ' Debug use only
    Private Sub Triangle()
        Gl.glBegin(Gl.GL_TRIANGLES)
        Gl.glColor3f(1, 0, 0) : Gl.glVertex3f(0, 0, 1)
        Gl.glColor3f(0, 1, 0) : Gl.glVertex3f(1, 0, 0)
        Gl.glColor3f(0, 0, 1) : Gl.glVertex3f(0, 1, 0)
        Gl.glEnd()
    End Sub

    ' OpenGL commands to draw a sphere
    Private Sub Sphere(ByVal center As Coordinate, ByVal r As Single)
        Dim i, j As Integer
        Dim lng As Double
        Dim x, y As Double
        Dim sections As Integer = 4

        For i = 0 To sections
            Dim lat0 As Double = Math.PI * (-0.5 + (i - 1) / sections)
            Dim z0 As Double = Sin(lat0)
            Dim zr0 As Double = Cos(lat0)

            Dim lat1 As Double = Math.PI * (-0.5 + i / sections)
            Dim z1 As Double = Sin(lat1)
            Dim zr1 As Double = Cos(lat1)

            Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, Gl.GL_LINE)
            'Gl.glBegin(Gl.GL_QUADS)
            Gl.glBegin(Gl.GL_QUAD_STRIP)
            For j = 0 To sections
                lng = 2 * Math.PI * (j - 1) / sections
                x = Cos(lng)
                y = Sin(lng)

                Gl.glNormal3f(-x * zr0, -y * zr0, -z0)
                Gl.glVertex3f(r * x * zr0 + center.x, r * y * zr0 + center.y, r * z0 + center.z)
                Gl.glNormal3f(-x * zr1, -y * zr1, -z1)
                Gl.glVertex3f(r * x * zr1 + center.x, r * y * zr1 + center.y, r * z1 + center.z)
            Next
            Gl.glEnd()
        Next
    End Sub

    ' OpenGL commands to set window size parameters
    Public Sub ResizeScene(ByVal w As Integer, ByVal h As Integer, ByVal zoom As Single)
        If h = 0 Then h = 1
        Gl.glViewport(0, 0, w, h)
        Gl.glMatrixMode(Gl.GL_PROJECTION)
        Gl.glLoadIdentity()
        Glu.gluPerspective(45.0, w / h, Abs(0.001 * zoom), Abs(5.0 * zoom))
        Gl.glMatrixMode(Gl.GL_MODELVIEW)
        'Gl.glLoadIdentity()
        'Application.DoEvents()
    End Sub

    ' Determine color based on normalized intensity values
    Private Function ColorCode(ByVal Intensity As Single, Optional ByVal Mode As Integer = 1) As Single()
        Dim Color(2) As Single

        If Intensity >= 0 Then
            Color(0) = 0
            Color(1) = 0
            Color(2) = 0

            Color(1) = Sin(3.14159 * (Intensity))
            If Mode = 1 Then
                Color(2) = Cos(3.14159 * (1 + Intensity))
                Color(0) = Cos(3.14159 * (Intensity))
            Else
                Color(0) = Cos(3.14159 * (1 + Intensity))
                Color(1) = Cos(3.14159 * (Intensity))
            End If
        Else
            Color(0) = 0.6
            Color(1) = 0.6
            Color(2) = 0.6
        End If

        ColorCode = Color
    End Function

    Private Function LightSettings(ByVal Light_Parameter As Single) As Single()()
        Dim Light_Settings(1)() As Single
        ReDim Light_Settings(0)(2)
        ReDim Light_Settings(1)(2)

        Light_Settings(0)(0) = Light_Parameter
        Light_Settings(0)(1) = Light_Parameter
        Light_Settings(0)(2) = Light_Parameter

        Light_Settings(1)(0) = 1 - Light_Parameter
        Light_Settings(1)(1) = 1 - Light_Parameter
        Light_Settings(1)(2) = 1 - Light_Parameter

        LightSettings = Light_Settings

    End Function
End Module
